import { extend } from 'quasar'
export default {
  props: {
    entity: String, // 当前对象的实体名（如：Page)
    linkMoreEntity: String, // 多对多中间关联表（如：LinkPageResource)
    otherEntity: String, // 关联的另外一方的从表（如：Resource）
    linkEntityMyKey: String, // 多对多表中当前表的字段名，默认：entity+'Id'
    linkMoreMyId: String, // 当前表的当前该数据的值（id）
    queryParam: {
      type: Object,
      default: () => {
        return {}
      }
    } // 对otherEntity查询列表的查询过滤条件
  },
  data() {
    return {
      queryParamInner: extend({}, this.queryParam)
    }
  },
  computed: {
    moduleInner() {
      return this.entity && this.entity.split('/')[1]
    },
    entityInner() {
      return this.entity && this.entity.split('/')[2]
    },
    linkEntityMyKeyInner() {
      return this.linkEntityMyKey ? this.linkEntityMyKey : `${this.entityInner}Id`
    },

    linkMoreModule() {
      return this.linkMoreEntity && this.linkMoreEntity.split('/')[1]
    },
    linkMoreTable() {
      return this.linkMoreEntity && this.linkMoreEntity.split('/')[2]
    },
    linkOtherModule() {
      return this.otherEntity && this.otherEntity.split('/')[1]
    },
    linkOtherTable() {
      return this.otherEntity && this.otherEntity.split('/')[2]
    }
  },
  methods: {
    checkModelAndEntity: function() {
      return this.entity
    },
    genBaseUrl: async function(methodName) {
      await this.checkModelAndEntity()

      if (!this.moduleInner) {
        console.error('this.module is null,', this.moduleInner)
      }
      if (!this.entityInner) {
        console.error('this.entity is null,', this.entityInner)
      }
      if (!methodName) {
        console.error('methodName is null,', methodName)
      }
      return '/' + this.moduleInner + '/' + this.entityInner + methodName
    },
    /**
     * 查询数据，使用关联表查询数据
     * @param {*} queryParam
     */
    async queryDataAtLinkEntity(queryParam) {
      queryParam && extend(this.queryParamInner, queryParam)
      const l = extend({}, this.queryParamInner)
      // 多选直接修改的情况
      if (this.linkMoreMyId && this.linkEntityMyKeyInner) {
        l['_' + this.linkEntityMyKeyInner] = this.linkMoreMyId
      }
      let url
      if (this.linkMoreEntity) {
        if (!this.linkMoreMyId) {
          throw new Error('未设置linkMoreMyId')
        }
        let linkTable = {}
        let linkFields = ['_id', 'name', 'title', this.titleKey, this.valueKey, 'pid']
        if (this.isGroup && this.groupKey) {
          linkFields.push(this.groupKey)
        }
        linkTable[this.linkOtherTable] = {
          type: 'inner join',
          fields: linkFields,
          linkFiled: '_id',
          myFiled: this.linkOtherTable + 'Id'
        }
        url = '/' + this.linkOtherModule + '/' + this.linkOtherTable + '/query?'
        l.linkTable = JSON.stringify(linkTable)
        l.fields = '_id,versionNum'
      } else {
        url = await this.genBaseUrl('/query?')
      }
      url += Object.keys(l)
        .map(k => encodeURIComponent(k) + '=' + encodeURIComponent(l[k]))
        .join('&')
      let response = await this.$axios.get(url)
      return response
    }
  },
  async mounted() {
    await this.checkModelAndEntity()
  }
}
