// uploadFile.js
import SparkMD5 from 'spark-md5'
import { SessionStorage } from 'quasar'

const checkByMd5Url = '/file/FileItem/getFileByMd5'
const checkChunkByMd5Url = '/file/FileItem/getChunkByMd5'
const uploadUrl = '/api/file/FileItem/uploadFiles'
const uploadByChunk = '/api/file/FileItem/uploadByChunk'
const chunkMaxSize = 2097152

// 对整个文件进行md5
let getFileMD5 = (file, callback) => {
  // 声明必要的变量
  let fileReader = new FileReader(),
    // 文件每块分割2M，计算分割详情
    chunkSize = chunkMaxSize,
    chunks = Math.ceil(file.size / chunkSize),
    currentChunk = 0,
    // 创建md5对象（基于SparkMD5）
    spark = new SparkMD5()

  // 每块文件读取完毕之后的处理
  fileReader.onload = e => {
    // 每块交由sparkMD5进行计算
    spark.appendBinary(e.target.result)
    currentChunk++

    // 如果文件处理完成计算MD5，如果还有分片继续处理
    if (currentChunk < chunks) {
      loadNext() // 递归调用下一片
    } else {
      callback(spark.end())
    }
  }

  // 处理单片文件的内容，然后加入到md5
  let loadNext = () => {
    let start = currentChunk * chunkSize,
      end = start + chunkSize >= file.size ? file.size : start + chunkSize
    fileReader.readAsBinaryString(file.slice(start, end))
  }
  loadNext()
}

let uploadFile = async function(file, endCallback, onprogress, onerror) {
  // console.log('vue:::', this)

  let uploadChunk = (file, currentChunk, endCallback, onprogress, error) => {
    let fileReader = new FileReader()
    // 上传文件块的大小，可自定义
    let chunkSize = chunkMaxSize
    // 计算改文件的可分为多少块
    let chunks = Math.ceil(file.size / chunkSize)
    let start, end
    // 文件切割后的回调，e.target.result为切割的文件块
    fileReader.onload = async e => {
      // console.log('upload e.target.result', e.target, e.target.result)
      // 获取单片文件的md5
      let s = new SparkMD5()
      s.appendBinary(e.target.result)
      // let md5 = SparkMD5.hashBinary(e.target.result)
      // spark = new SparkMD5()
      let md5 = s.end()
      // console.log(e.target, md5, s.end(), s.end())
      // 查询后台判断当前块文件有没有上传，如果已经上传则不需要上传，继续读取下一块
      let res = await this.$axios.get(checkChunkByMd5Url + '?_id=' + md5)
      let chunkIsExists = false
      if (res.data && res.data._id) {
        chunkIsExists = true
        // // 执行下一个上传
        // let p = parseInt((100 / chunks) * (currentChunk + 1))
        // onprogress && onprogress(p)
        // await loadNextChunk()
        // return
      }
      // 重新按二进制读取
      let fileReaderBuffer = new FileReader()
      fileReaderBuffer.readAsArrayBuffer(file.slice(start, end))
      fileReaderBuffer.onload = async e => {
        // 用FormData传输文件对象
        let fd = new FormData()
        // 设置文件上传接口的需要的参数
        fd.append('_id', file._id)
        fd.append('md5', file.md5)
        fd.append('chunkMd5', md5)
        fd.append('size', file.size + '')
        fd.append('name', file.name)
        fd.append('type', file.type)
        fd.append('extendName', file.extendName)
        fd.append('chunks', chunks)
        fd.append('currentChunk', currentChunk)
        if (!chunkIsExists) {
          // 设置上传的当前的文件块
          fd.append('fileObj', new Blob([e.target.result]))
        }

        let xhr = new XMLHttpRequest()
        xhr.open('post', (window.fileApiBasePath || '') + (chunkIsExists ? uploadByChunk : uploadUrl), true)
        xhr.setRequestHeader(
          'token',
          file.token || SessionStorage.getItem('___user_token')
        )
        xhr.onreadystatechange = async () => {
          // 上传成功
          if (xhr.readyState === 4 && xhr.status === 200) {
            if (currentChunk < chunks) {
              await loadNextChunk() // 继续切割下一块文件
            } else {
              // 文件上传成功
              endCallback && endCallback()
            }
            xhr = null
            return
          }
          if (xhr.readyState === 4 && xhr.status >= 400) {
            if (error) {
              error({ status: xhr.status })
            }
            // // 文件上传出错,提示用户重新上传
            // if (currentChunk < chunks) {
            //   await loadNextChunk() // 继续切割下一块文件
            // } else {
            //   // 文件上传成功
            //   endCallback && endCallback()
            // }
          }
        }
        // 文件上传进度条
        xhr.upload.onprogress = e => {
          // 计算上传的进度
          // const progress = parseInt(((e.loaded + currentChunk * chunkSize) / file.size) * 100)
          // 更新ui
          // 计算出总的百分比的数字0-100以内
          let p = parseInt(
            (100 / chunks) * (currentChunk + 1) * (e.loaded / e.total)
          )
          p = p > 100 ? 100 : p
          p = p < 1 ? 1 : p
          onprogress && onprogress(p)
        }
        xhr.onerror = xhr.upload.onerror = err => {
          // 文件上传出错
          if (error) {
            error(err)
          }
        }
        // 开始发送
        xhr.send(fd)
        fd = null
      }
    }

    // 处理单片文件的上传
    let loadNextChunk = async () => {
      if (currentChunk < chunks) {
        start = currentChunk * chunkSize
        end = Math.min(start + chunkSize, file.size)
        // 切割文件并触发fileReader.onload，这就是处理文件分片的关键代码
        // fileReader.readAsArrayBuffer
        fileReader.readAsBinaryString(file.slice(start, end))
        currentChunk++
      } else {
        endCallback && endCallback()
      }
    }
    // 触发文件第一块上传
    loadNextChunk()
  }

  // 得到md5码
  getFileMD5(file, async md5 => {
    file._id || (file._id = ''.genUUID())
    // 存储文件的md5码
    file.md5 = md5
    // 拿md5码查询后台数据库是否存在此md5码，如果存在则无需上传
    // handleAjax为封装好的ajax方法
    let res = await this.$axios.get(checkByMd5Url + '?md5=' + md5)
    if (!res.data || !res.data.md5) {
      // 不存在,开始上传文件
      uploadChunk(file, 0, endCallback, onprogress, onerror)
    } else {
      onprogress && onprogress(99)
      // 秒传,文件上传成功
      endCallback && endCallback(res.data)
    }
  })
}
export default uploadFile
