const required = require('./RequiredRule').default
const minLength = require('./MinLengthRule').default
const maxLength = require('./MaxLengthRule').default
const maxDecimalDigits = require('./MaxDecimalDigitsRule').default
const minValue = require('./MinValueRule').default
const maxValue = require('./MaxValueRule').default
const phone = require('./PhoneRule.js').default
const idCard = require('./IdCardRule.js').default
const email = require('./EmailRule.js').default
export default {
  required,
  minLength,
  maxLength,
  maxDecimalDigits,
  minValue,
  maxValue,
  phone,
  idCard,
  email,
  execute({ rules, rule, field, value }) {
    if (!rules) {
      if (!rule) {
        console.warn('-- rule: not rules')
        return
      }
      if (typeof rule === 'object') {
        rules = [rule]
      } else {
        rules = rule.split(',')
      }
    }

    for (const r of rules) {
      const ruleParams = { field, value }
      if (typeof r === 'object') {
        Object.assign(ruleParams, r)
      } else {
        ruleParams.name = r
      }
      const ruleImpl = this[ruleParams.name] && this[ruleParams.name](ruleParams)
      if (!ruleImpl) {
        throw new Error(`[${ruleParams.name}]未定义`)
      }
      if (!ruleImpl.execute()) {
        throw new Error(`[${field.title || field.name}]${ruleImpl.errorMsg || ruleParams.name}`)
      }
    }
  }
}
